clear; close all

y_shift = 1;
y_scale = 10;
sample_num = 1e4;

q = 0.5;
p = [q 1-2*q q];

fname_penalty = 'penalty_gaussian';
fname_penalty_sum = 'penalty_sum';

fname_expected_penalty = 'expected_penalty_discrete';
fname_expected_sum = 'expected_sum_discrete';

dist_matrix = p;
possible_y = round([y_shift y_shift+y_scale y_shift+2*y_scale]);
parameters = [possible_y; dist_matrix];

max_n = 200;
n = 1:max_n;
coefficient = 0.9;
inf_max = 1000;

%Lookup tables to precalculate most stuff.
age_penalty_lut = [-inf_max feval(fname_penalty, n, coefficient)];
expected_sum_lut = expected_sum_generate_lut(age_penalty_lut, max_n);

[~,samples] = histc(rand(1,sample_num+1),cumsum(p(:))/sum(p)); % discrete distributions of 0,1,2
samples = samples'*y_scale+y_shift;
mean_ = mean(samples);

f_max_values = [0 0.04:0.0001:0.2];

age_uniform_wait=zeros(1,length(f_max_values));
age_zero_wait=NaN(1,length(f_max_values));
age_optimal=zeros(1,length(f_max_values));
wait_times_uniform = zeros(1, sample_num);
wait_times_zero = zeros(1, sample_num);
wait_times_optimal = zeros(1, sample_num);
k = 1; %age at k = 0 will always be 0 anyway.
for f_max = f_max_values(2:end)
    fprintf("f_max = %.3f\n", f_max);
    T = min(1/f_max, inf_max);
    k = k + 1;
    
    %% Uniform Sampling
    wait_times_uniform = ceil(ones(1,sample_num)*T);
    S_i = [0 cumsum(wait_times_uniform)];
    D_i = zeros(1,sample_num+1);
    D_i(1) = S_i(1)+samples(1);
    for i = 1:sample_num
        %Need the extra computation to account for the queueing delay.
        D_i(i+1) = max(S_i(i+1), D_i(i)) + samples(i+1);
    end
    Y_i = D_i(1:sample_num) - S_i(1:sample_num);
    Z_plus_Y_ip1 = D_i(2:sample_num+1) - D_i(1:sample_num);
    Expectedq = mean(feval(fname_penalty_sum, expected_sum_lut, Y_i, Z_plus_Y_ip1));
%     Expectedq_old = mean(feval('q_mutual_info_binary_old', age_penalty_lut, Y_i, zeros(1,sample_num), Z_plus_Y_ip1));
    age_uniform_wait(k) = Expectedq/(mean(D_i(2:sample_num+1) - D_i(1:sample_num)));
    
    %% Zero Wait
    %Zero wait only works when 1/f_max > E[Y]
    if(T < mean_)
        S_i = [0 cumsum(samples(1:sample_num)')];
        D_i = cumsum(samples');
        Y_i = D_i(1:sample_num) - S_i(1:sample_num);
        Z_plus_Y_ip1 = D_i(2:sample_num+1) - D_i(1:sample_num);
        Expectedq = mean(feval(fname_penalty_sum, expected_sum_lut, Y_i, Z_plus_Y_ip1));
        age_zero_wait(k) = Expectedq/(mean(D_i(2:sample_num+1) - D_i(1:sample_num))); %E[sum(p(n-Si))]/E[Di+1-Di]
    end
    
    %% Optimal Policy
    optimal_policy_mutual_info;
    S_i = [0 cumsum(wait_times_optimal+samples(1:sample_num)')];
    D_i = zeros(1,sample_num+1);
    D_i(1) = S_i(1)+samples(1);
    for i = 1:sample_num
        %Need the extra computation to account for the queueing delay.
        D_i(i+1) = max(S_i(i+1), D_i(i)) + samples(i+1);
    end
    Y_i = D_i(1:sample_num) - S_i(1:sample_num);
    Z_plus_Y_ip1 = D_i(2:sample_num+1) - D_i(1:sample_num);
    Expectedq = mean(feval(fname_penalty_sum, expected_sum_lut, Y_i, Z_plus_Y_ip1));
    age_optimal(k) = Expectedq/(mean(D_i(2:sample_num+1) - D_i(1:sample_num))); %E[sum(p(n-Si))]/E[Di+1-Di]
end
%Mutual Information Graph

color_index = 1;
plot(f_max_values,-age_optimal,'k','linewidth',2);
hold on;
legendInfo{color_index} = 'Optimal Policy'; 

color_index = color_index + 1;
plot(f_max_values,-age_zero_wait,'r:','linewidth',2);
hold on;
legendInfo{color_index} = 'Zero Wait';

color_index = color_index + 1;
plot(f_max_values,-age_uniform_wait,'b-.','linewidth',2);
hold on;
legendInfo{color_index} = 'Uniform Sampling';

axis([0 0.18 0 0.16])


%axis([T_min T_max -0.1 0])
xlabel('f_{max}', 'FontSize', 16);
ylabel('Average Mutual Information', 'FontSize', 16);
legend(legendInfo, 'FontSize', 16);
saveas(figure(1),'figs/figure7_constrained_gaussian_discrete.fig');

%Penalty Graph
figure;
color_index = 1;
plot(f_max_values,age_uniform_wait,'b-.','linewidth',2);
hold on;
legendInfo{color_index} = 'Uniform Sampling';

color_index = color_index + 1;
plot(f_max_values,age_zero_wait,'r:','linewidth',2);
hold on;
legendInfo{color_index} = 'Zero Wait';

color_index = color_index + 1;
plot(f_max_values,age_optimal,'k','linewidth',2);
hold on;
legendInfo{color_index} = 'Optimal Policy'; 

%axis([T_min T_max -0.1 0])
xlabel('f_{max}', 'FontSize', 16);
ylabel('Average Age Penalty', 'FontSize', 16);
legend(legendInfo, 'FontSize', 16);
saveas(figure(2),'figs/figure7_1_constrained_gaussian_discrete_penalty.fig');

